@@ -53,21 +53,19 @@
 #include "ipcalc.h"
 #include "scheduler.h"
 #include "log.h"
+#include "routing_table.h"
 
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <sys/time.h>
 #include <net/route.h>
-#ifdef linux
-#include <linux/in_route.h>
-#endif
 #include <unistd.h>
 #include <errno.h>
 #include <time.h>
-#ifndef WIN32
+#ifndef _WIN32
 #include <pthread.h>
-#else
+#else /* _WIN32 */
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #undef interface
@@ -87,7 +85,7 @@ struct ThreadPara {
   void *(*Func) (void *);
   void *Arg;
 };
-#endif
+#endif /* _WIN32 */
 
 static int hna_check_interval	= DEFAULT_HNA_CHECK_INTERVAL;
 /* set default interval, in case none is given in the config file */
@@ -470,7 +468,7 @@ update_routing(void)
       continue;
     }
     
-    if ((iflags & RTF_UP) && (metric == 0)) {
+    if ((iflags & RTF_UP) && (metric != RT_METRIC_DEFAULT)) {
       hna->checked = true;
     }
   }
@@ -624,7 +622,7 @@ add_to_hna_group(struct hna_group *list_root)
 }
 
 
-#ifdef WIN32
+#ifdef _WIN32
 
 /*
  * Windows pthread compat stuff
@@ -708,7 +706,7 @@ pthread_mutex_unlock(HANDLE * Hand)
   return 0;
 }
 
-#endif
+#endif /* _WIN32 */
 
 /*
  * Local Variables:
