@@ -23,16 +23,16 @@
 
 #ifdef  __cplusplus
 extern "C" {
-#endif
+#endif /* __cplusplus */
 
-int nmea_calc_crc(const char *buff, int buff_sz);
-int nmea_atoi(const char *str, int str_sz, int radix);
-double nmea_atof(const char *str, int str_sz);
-int nmea_printf(char *buff, int buff_sz, const char *format, ...) __attribute__ ((format(printf, 3, 4)));
-int nmea_scanf(const char *buff, int buff_sz, const char *format, ...);
+int nmea_calc_crc(const char *s, const int len);
+int nmea_atoi(const char *s, const int len, const int radix);
+double nmea_atof(const char *s, const int len);
+int nmea_printf(char *s, int len, const char *format, ...) __attribute__ ((format(printf, 3, 4)));
+int nmea_scanf(const char *s, int len, const char *format, ...);
 
 #ifdef  __cplusplus
 }
-#endif
+#endif /* __cplusplus */
 
 #endif /* __NMEA_TOK_H__ */
