@@ -55,11 +55,12 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#ifdef linux
+#ifdef __linux__
 #include <linux/types.h>
 #include <linux/rtnetlink.h>
 #include <linux/version.h>
-#endif
+#include <sys/stat.h>
+#endif /* __linux__ */
 
 extern FILE *yyin;
 extern int yyparse(void);
@@ -120,11 +121,11 @@ main(int argc, char *argv[])
   return 0;
 }
 
-#else
+#else /* MAKEBIN */
 
 /* Build as part of olsrd */
 
-#endif
+#endif /* MAKEBIN */
 
 int
 olsrd_parse_cnf(const char *filename)
@@ -210,7 +211,7 @@ olsrd_print_interface_cnf(struct if_config_options *cnf, struct if_config_option
   printf("\tAutodetect changes       : %s%s\n", cnf->autodetect_chg ? "yes" : "no",DEFAULT_STR(autodetect_chg));
 }
 
-#ifdef linux
+#ifdef __linux__
 static int olsrd_sanity_check_rtpolicy(struct olsrd_config *cnf) {
   int prio;
 
@@ -346,7 +347,7 @@ static int olsrd_sanity_check_rtpolicy(struct olsrd_config *cnf) {
   return 0;
 }
 
-#endif
+#endif /* __linux__ */
 
 
 static 
@@ -521,24 +522,24 @@ olsrd_sanity_check_cnf(struct olsrd_config *cnf)
     return -1;
   }
 
-#if defined linux
+#ifdef __linux__
 #if !defined LINUX_VERSION_CODE || !defined KERNEL_VERSION
 #error "Both LINUX_VERSION_CODE and KERNEL_VERSION need to be defined"
-#else
+#else /* !defined LINUX_VERSION_CODE || !defined KERNEL_VERSION */
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,24)
   if (cnf->ip_version == AF_INET6 && cnf->smart_gw_active) {
     fprintf(stderr, "Smart gateways are not supported for linux kernel < 2.6.24 and ipv6\n");
     return -1;
   }
-#endif
-#endif
+#endif /* LINUX_VERSION_CODE < KERNEL_VERSION(2,6,24) */
+#endif /* !defined LINUX_VERSION_CODE || !defined KERNEL_VERSION */
 
   /* this rtpolicy settings are also currently only used in Linux */
   if (olsrd_sanity_check_rtpolicy(cnf)) {
     return -1;
   }
 
-#endif
+#endif /* __linux__ */
 
   if (in == NULL) {
     fprintf(stderr, "No interfaces configured!\n");
@@ -553,6 +554,113 @@ olsrd_sanity_check_cnf(struct olsrd_config *cnf)
 	  fprintf(stderr, "Warning, you are using the min_tc_vtime hack. We hope you know what you are doing... contact olsr.org otherwise.\n");
   }
 
+#ifdef __linux__
+  if ((cnf->smart_gw_use_count < MIN_SMARTGW_USE_COUNT_MIN) || (cnf->smart_gw_use_count > MAX_SMARTGW_USE_COUNT_MAX)) {
+    fprintf(stderr, "Error, bad gateway use count %d, outside of range [%d, %d]\n",
+        cnf->smart_gw_use_count, MIN_SMARTGW_USE_COUNT_MIN, MAX_SMARTGW_USE_COUNT_MAX);
+    return -1;
+  }
+
+  if (cnf->smart_gw_use_count > 1) {
+    struct sgw_egress_if * sgwegressif = cnf->smart_gw_egress_interfaces;
+
+	/* check that the sgw takedown percentage is in the range [0, 100] */
+	if (/*(cnf->smart_gw_takedown_percentage < 0) ||*/ (cnf->smart_gw_takedown_percentage > 100)) {
+	  fprintf(stderr, "Error, smart gateway takedown percentage (%u) is not in the range [0, 100]\n",
+		  cnf->smart_gw_takedown_percentage);
+	  return -1;
+	}
+
+    if (!cnf->smart_gw_policyrouting_script) {
+      fprintf(stderr, "Error, no policy routing script configured in multi-gateway mode\n");
+      return -1;
+    }
+
+    {
+      struct stat statbuf;
+
+      int r = stat(cnf->smart_gw_policyrouting_script, &statbuf);
+      if (r) {
+        fprintf(stderr, "Error, policy routing script not found: %s\n", strerror(errno));
+        return -1;
+      }
+
+      if (!S_ISREG(statbuf.st_mode)) {
+        fprintf(stderr, "Error, policy routing script not a regular file\n");
+        return -1;
+      }
+
+      if (statbuf.st_uid) {
+        fprintf(stderr, "Error, policy routing script must be owned by root\n");
+        return -1;
+      }
+
+      if (!(statbuf.st_mode & (S_IRUSR | S_IXUSR))) {
+        fprintf(stderr, "Error, policy routing script is not executable\n");
+        return -1;
+      }
+    }
+
+    /* egress interface(s) must be set */
+    if (!sgwegressif) {
+      fprintf(stderr, "Error, no egress interfaces configured in multi-gateway mode\n");
+      return -1;
+    }
+
+    /* an egress interface must not be an OLSR interface */
+    while (sgwegressif) {
+      struct olsr_if * olsrif = cnf->interfaces;
+      while (olsrif) {
+        if (!strcmp(olsrif->name, sgwegressif->name)) {
+          fprintf(stderr, "Error, egress interface %s already is an OLSR interface\n", sgwegressif->name);
+          return -1;
+        }
+        olsrif = olsrif->next;
+      }
+      cnf->smart_gw_egress_interfaces_count++;
+      sgwegressif = sgwegressif->next;
+    }
+
+    if (cnf->smart_gw_egress_interfaces_count > MAX_SMARTGW_EGRESS_INTERFACE_COUNT_MAX) {
+      fprintf(stderr, "Error, egress interface count %u not in range [1, %u]\n",
+          cnf->smart_gw_egress_interfaces_count, MAX_SMARTGW_EGRESS_INTERFACE_COUNT_MAX);
+      return -1;
+    }
+
+    {
+      uint8_t egressLow = cnf->smart_gw_mark_offset_egress;
+      uint8_t egressHigh = egressLow + cnf->smart_gw_egress_interfaces_count - 1;
+      uint8_t tunnelsLow = cnf->smart_gw_mark_offset_tunnels;
+      uint8_t tunnelsHigh = tunnelsLow + cnf->smart_gw_use_count - 1;
+      bool overlap = false;
+
+      /* check that the egress interface marks range does not overflow */
+      if (egressLow > (UINT8_MAX - cnf->smart_gw_egress_interfaces_count)) {
+        fprintf(stderr, "Error, egress interface mark offset %u together with egress interface count %u overflows range [0, %u]\n",
+            egressLow, cnf->smart_gw_egress_interfaces_count, UINT8_MAX);
+        return -1;
+      }
+
+      /* check that the tunnel interface marks range does not overflow */
+      if (tunnelsLow > (UINT8_MAX - cnf->smart_gw_use_count)) {
+        fprintf(stderr, "Error, tunnel interface mark offset %u together with use count %u overflows range [0, %u]\n",
+            tunnelsLow, cnf->smart_gw_use_count, UINT8_MAX);
+        return -1;
+      }
+
+      /* check that the egress and tunnel marks ranges do not overlap */
+      overlap =            ((tunnelsLow <= egressLow)   && (egressLow   <= tunnelsHigh));
+      overlap = overlap || ((tunnelsLow <= egressHigh)  && (egressHigh  <= tunnelsHigh));
+      overlap = overlap || ((egressLow  <= tunnelsLow)  && (tunnelsLow  <= egressHigh));
+      overlap = overlap || ((egressLow  <= tunnelsHigh) && (tunnelsHigh <= egressHigh));
+      if (overlap) {
+        fprintf(stderr, "Error, egress interface mark range [%u, %u] overlaps with tunnel interface mark range [%u, %u]\n",
+            egressLow, egressHigh, tunnelsLow, tunnelsHigh);
+        return -1;
+      }
+    }
+  }
+
   if (cnf->smart_gw_period < MIN_SMARTGW_PERIOD || cnf->smart_gw_period > MAX_SMARTGW_PERIOD) {
     fprintf(stderr, "Error, bad gateway period: %d msec (should be %d-%d)\n",
         cnf->smart_gw_period, MIN_SMARTGW_PERIOD, MAX_SMARTGW_PERIOD);
@@ -569,6 +677,13 @@ olsrd_sanity_check_cnf(struct olsrd_config *cnf)
     return -1;
   }
 
+  /* no checks are needed on:
+   * - cnf->smart_gw_weight_exitlink_up
+   * - cnf->smart_gw_weight_exitlink_down
+   * - cnf->smart_gw_weight_etx
+   * - cnf->smart_gw_divider_etx
+   */
+
   if (cnf->smart_gw_type >= GW_UPLINK_CNT) {
     fprintf(stderr, "Error, illegal gateway uplink type: %d\n", cnf->smart_gw_type);
     return -1;
@@ -583,6 +698,7 @@ olsrd_sanity_check_cnf(struct olsrd_config *cnf)
         cnf->smart_gw_uplink, MIN_SMARTGW_SPEED, MAX_SMARTGW_SPEED);
     return -1;
   }
+#endif /* __linux__ */
 
   if (cnf->interface_defaults == NULL) {
     /* get a default configuration if the user did not specify one */
@@ -596,6 +712,9 @@ olsrd_sanity_check_cnf(struct olsrd_config *cnf)
   while (in) {
     struct olsr_lq_mult *mult, *mult_orig;
 
+    assert(in->cnf);
+    assert(in->cnfi);
+
     io = in->cnf;
 
     olsrd_print_interface_cnf(in->cnf, in->cnfi, false);
@@ -611,8 +730,6 @@ olsrd_sanity_check_cnf(struct olsrd_config *cnf)
       /*save interface specific lqmults, as they are merged togehter later*/
       mult_orig = io->lq_mult;
 
-      assert(in->cnf);
-      assert(in->cnfi);
       for (pos = 0; pos < sizeof(*in->cnf); pos++) {
         if (cnfptr[pos] != cnfiptr[pos]) {
           cnfptr[pos] = defptr[pos]; cnfiptr[pos]=0x00;
@@ -728,6 +845,7 @@ set_default_cnf(struct olsrd_config *cnf)
 
   cnf->debug_level = DEF_DEBUGLVL;
   cnf->no_fork = false;
+  cnf->pidfile = NULL;
   cnf->host_emul = false;
   cnf->ip_version = AF_INET;
   cnf->ipsize = sizeof(struct in_addr);
@@ -777,10 +895,22 @@ set_default_cnf(struct olsrd_config *cnf)
   cnf->niit6to4_if_index = 0;
 
   cnf->smart_gw_active = DEF_SMART_GW;
+  cnf->smart_gw_always_remove_server_tunnel = DEF_SMART_GW_ALWAYS_REMOVE_SERVER_TUNNEL;
+  cnf->smart_gw_use_count = DEF_GW_USE_COUNT;
+  cnf->smart_gw_takedown_percentage = DEF_GW_TAKEDOWN_PERCENTAGE;
+  cnf->smart_gw_policyrouting_script = NULL;
+  cnf->smart_gw_egress_interfaces = NULL;
+  cnf->smart_gw_egress_interfaces_count = 0;
+  cnf->smart_gw_mark_offset_egress = DEF_GW_MARK_OFFSET_EGRESS;
+  cnf->smart_gw_mark_offset_tunnels = DEF_GW_MARK_OFFSET_TUNNELS;
   cnf->smart_gw_allow_nat = DEF_GW_ALLOW_NAT;
   cnf->smart_gw_period = DEF_GW_PERIOD;
   cnf->smart_gw_stablecount = DEF_GW_STABLE_COUNT;
   cnf->smart_gw_thresh = DEF_GW_THRESH;
+  cnf->smart_gw_weight_exitlink_up = DEF_GW_WEIGHT_EXITLINK_UP;
+  cnf->smart_gw_weight_exitlink_down = DEF_GW_WEIGHT_EXITLINK_DOWN;
+  cnf->smart_gw_weight_etx = DEF_GW_WEIGHT_ETX;
+  cnf->smart_gw_divider_etx = DEF_GW_DIVIDER_ETX;
   cnf->smart_gw_type = DEF_GW_TYPE;
   cnf->smart_gw_uplink = DEF_UPLINK_SPEED;
   cnf->smart_gw_uplink_nat = DEF_GW_UPLINK_NAT;
@@ -789,17 +919,17 @@ set_default_cnf(struct olsrd_config *cnf)
   cnf->use_src_ip_routes = DEF_USE_SRCIP_ROUTES;
   cnf->set_ip_forward = true;
 
-#ifdef linux
+#ifdef __linux__
   cnf->rtnl_s = 0;
-#endif
+#endif /* __linux__ */
 
 #if defined __FreeBSD__ || defined __FreeBSD_kernel__ || defined __APPLE__ || defined __NetBSD__ || defined __OpenBSD__
   cnf->rts = 0;
-#endif
+#endif /* defined __FreeBSD__ || defined __FreeBSD_kernel__ || defined __APPLE__ || defined __NetBSD__ || defined __OpenBSD__ */
 
 #ifdef HTTPINFO_PUD
   cnf->pud_position = NULL;
-#endif
+#endif /* HTTPINFO_PUD */
 }
 
 struct if_config_options *
@@ -901,6 +1031,28 @@ olsrd_print_cnf(struct olsrd_config *cnf)
 
   printf("Smart Gateway    : %s\n", cnf->smart_gw_active ? "yes" : "no");
 
+  printf("SmGw. Del Srv Tun: %s\n", cnf->smart_gw_always_remove_server_tunnel ? "yes" : "no");
+
+  printf("SmGw. Use Count  : %u\n", cnf->smart_gw_use_count);
+
+  printf("SmGw. Takedown%%  : %u\n", cnf->smart_gw_takedown_percentage);
+
+  printf("SmGw. Pol. Script: %s\n", cnf->smart_gw_policyrouting_script);
+
+  printf("SmGw. Egress I/Fs:");
+  {
+    struct sgw_egress_if * sgwegressif = cnf->smart_gw_egress_interfaces;
+    while (sgwegressif) {
+      printf(" %s", sgwegressif->name);
+      sgwegressif = sgwegressif->next;
+    }
+  }
+  printf("\n");
+
+  printf("SmGw. Mark Egress: %u\n", cnf->smart_gw_mark_offset_egress);
+
+  printf("SmGw. Mark Tunnel: %u\n", cnf->smart_gw_mark_offset_tunnels);
+
   printf("SmGw. Allow NAT  : %s\n", cnf->smart_gw_allow_nat ? "yes" : "no");
 
   printf("SmGw. period     : %d\n", cnf->smart_gw_period);
@@ -978,7 +1130,7 @@ olsrd_print_cnf(struct olsrd_config *cnf)
   }
 }
 
-#if defined WIN32
+#if defined _WIN32
 struct ioinfo {
   unsigned int handle;
   unsigned char attr;
@@ -1021,7 +1173,7 @@ win32_olsrd_free(void *ptr)
 {
   free(ptr);
 }
-#endif
+#endif /* defined _WIN32 */
 
 static void update_has_gateway_fields(void) {
   struct ip_prefix_list *h;
