@@ -31,5 +31,5 @@ void olsr_remove_duplicate_handler(struct duplicate_handler *);
 void olsr_test_originator_collision(uint8_t msgType, uint16_t seqno);
 void olsr_handle_hna_collision(union olsr_ip_addr *hna, union olsr_ip_addr *orig);
 void olsr_handle_mid_collision(union olsr_ip_addr *mid, union olsr_ip_addr *orig);
-#endif
+#endif /* NO_DUPLICATE_DETECTION_HANDLER */
 #endif /* DUPLICATE_HANDLER_H_ */
