@@ -39,6 +39,8 @@
  *
  */
 
+#ifdef __linux__
+
 #include "kernel_tunnel.h"
 #include "kernel_routes.h"
 #include "log.h"
@@ -57,15 +59,18 @@
 #include <linux/ip.h>
 #include <linux/if_tunnel.h>
 #include <linux/version.h>
-#if defined linux
+
 #if !defined LINUX_VERSION_CODE || !defined KERNEL_VERSION
-#error "Both LINUX_VERSION_CODE and KERNEL_VERSION need to be defined"
-#else
+  #error "Both LINUX_VERSION_CODE and KERNEL_VERSION need to be defined"
+#endif /* !defined LINUX_VERSION_CODE || !defined KERNEL_VERSION */
+
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
+  #define LINUX_IPV6_TUNNEL
+#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24) */
+
+#ifdef LINUX_IPV6_TUNNEL
 #include <linux/ip6_tunnel.h>
-#endif
-#endif
-#endif
+#endif /* LINUX_IPV6_TUNNEL */
 
 //ifup includes
 #include <sys/socket.h>
@@ -73,16 +78,11 @@
 #include <sys/types.h>
 #include <net/if.h>
 
-static const char DEV_IPV4_TUNNEL[IFNAMSIZ] = TUNNEL_ENDPOINT_IF;
-static const char DEV_IPV6_TUNNEL[IFNAMSIZ] = TUNNEL_ENDPOINT_IF6;
-
 static bool store_iptunnel_state;
 static struct olsr_cookie_info *tunnel_cookie;
 static struct avl_tree tunnel_tree;
 
-int olsr_os_init_iptunnel(void) {
-  const char *dev = olsr_cnf->ip_version == AF_INET ? DEV_IPV4_TUNNEL : DEV_IPV6_TUNNEL;
-
+int olsr_os_init_iptunnel(const char * dev) {
   tunnel_cookie = olsr_alloc_cookie("iptunnel", OLSR_COOKIE_TYPE_MEMORY);
   olsr_cookie_set_memory_size(tunnel_cookie, sizeof(struct olsr_iptunnel_entry));
   avl_init(&tunnel_tree, avl_comp_default);
@@ -98,7 +98,7 @@ int olsr_os_init_iptunnel(void) {
   return olsr_os_ifip(if_nametoindex(dev), &olsr_cnf->main_addr, true);
 }
 
-void olsr_os_cleanup_iptunnel(void) {
+void olsr_os_cleanup_iptunnel(const char * dev) {
   while (tunnel_tree.count > 0) {
     struct olsr_iptunnel_entry *t;
 
@@ -108,141 +108,101 @@ void olsr_os_cleanup_iptunnel(void) {
 
     olsr_os_del_ipip_tunnel(t);
   }
-  if (!store_iptunnel_state) {
-    olsr_if_set_state(olsr_cnf->ip_version == AF_INET ? DEV_IPV4_TUNNEL : DEV_IPV6_TUNNEL, false);
+  if (olsr_cnf->smart_gw_always_remove_server_tunnel || !store_iptunnel_state) {
+    olsr_if_set_state(dev, false);
   }
 
   olsr_free_cookie(tunnel_cookie);
 }
 
 /**
- * creates an ipip tunnel (for ipv4)
+ * creates an ipip tunnel (for ipv4 or ipv6)
+ *
  * @param name interface name
- * @param target pointer to tunnel target IP, NULL if tunnel should be removed
+ * @param target pointer to tunnel target IP, NULL if tunnel should be removed.
+ * Must be of type 'in_addr_t *' for ipv4 and of type 'struct in6_addr *' for
+ * ipv6
  * @return 0 if an error happened,
  *   if_index for successful created tunnel, 1 for successful deleted tunnel
  */
-static int os_ip4_tunnel(const char *name, in_addr_t *target)
-{
-  struct ifreq ifr;
-  int err;
-  struct ip_tunnel_parm p;
-  char buffer[INET6_ADDRSTRLEN];
+static int os_ip_tunnel(const char *name, void *target) {
+	struct ifreq ifr;
+	int err;
+	void * p;
+	char buffer[INET6_ADDRSTRLEN];
+	const char * tunName;
+	struct ip_tunnel_parm p4;
+#ifdef LINUX_IPV6_TUNNEL
+	struct ip6_tnl_parm p6;
+#endif /* LINUX_IPV6_TUNNEL */
 
-  /* only IPIP tunnel if OLSR runs with IPv4 */
-  assert (olsr_cnf->ip_version == AF_INET);
-  memset(&p, 0, sizeof(p));
-  p.iph.version = 4;
-  p.iph.ihl = 5;
-  p.iph.ttl = 64;
-  p.iph.protocol = IPPROTO_IPIP;
-  if (target) {
-    p.iph.daddr = *target;
-  }
-  strncpy(p.name, name, IFNAMSIZ);
+	assert (name != NULL);
 
-  memset(&ifr, 0, sizeof(ifr));
-  strncpy(ifr.ifr_name, target != NULL ? DEV_IPV4_TUNNEL : name, IFNAMSIZ);
-  ifr.ifr_ifru.ifru_data = (void *) &p;
+	memset(&ifr, 0, sizeof(ifr));
 
-  if ((err = ioctl(olsr_cnf->ioctl_s, target != NULL ? SIOCADDTUNNEL : SIOCDELTUNNEL, &ifr))) {
-    olsr_syslog(OLSR_LOG_ERR, "Cannot %s a tunnel %s to %s: %s (%d)\n",
-        target != NULL ? "add" : "remove", name,
-        target != NULL ? inet_ntop(olsr_cnf->ip_version, target, buffer, sizeof(buffer)) : "-",
-        strerror(errno), errno);
-    return 0;
-  }
-  olsr_syslog(OLSR_LOG_INFO, "Tunnel %s %s, to %s",
-      name,
-      target != NULL ? "added" : "removed",
-      target != NULL ? inet_ntop(olsr_cnf->ip_version, target, buffer, sizeof(buffer)) : "-");
-  return target != NULL ? if_nametoindex(name) : 1;
-}
+	if (olsr_cnf->ip_version == AF_INET) {
+		p = &p4;
+		tunName = TUNNEL_ENDPOINT_IF;
 
-/**
- * creates an ipip tunnel (for ipv6)
- * @param name interface name
- * @param target pointer to tunnel target IP, NULL if tunnel should be removed
- * @return 0 if an error happened,
- *   if_index for successful created tunnel, 1 for successful deleted tunnel
- */
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
-static int os_ip6_tunnel(const char *name, struct in6_addr *target)
-{
-  struct ifreq ifr;
-  int err;
-  struct ip6_tnl_parm p;
+		memset(&p4, 0, sizeof(p4));
+		p4.iph.version = 4;
+		p4.iph.ihl = 5;
+		p4.iph.ttl = 64;
+		p4.iph.protocol = IPPROTO_IPIP;
+		if (target) {
+			p4.iph.daddr = *((in_addr_t *) target);
+		}
+		strscpy(p4.name, name, IFNAMSIZ);
+	} else {
+#ifdef LINUX_IPV6_TUNNEL
+		p = (void *) &p6;
+		tunName = TUNNEL_ENDPOINT_IF6;
 
-  /* only IP6 tunnel if OLSR runs with IPv6 */
-  assert (olsr_cnf->ip_version == AF_INET6);
-  memset(&p, 0, sizeof(p));
-  p.proto = 0; /* any protocol */
-  if (target) {
-    p.raddr = *target;
-  }
-  strncpy(p.name, name, IFNAMSIZ);
+		memset(&p6, 0, sizeof(p6));
+		p6.proto = 0; /* any protocol */
+		if (target) {
+			p6.raddr = *((struct in6_addr *) target);
+		}
+		strscpy(p6.name, name, IFNAMSIZ);
+#else /* LINUX_IPV6_TUNNEL */
+		return 0;
+#endif /* LINUX_IPV6_TUNNEL */
+	}
 
-  memset(&ifr, 0, sizeof(ifr));
-  strncpy(ifr.ifr_name, target != NULL ? DEV_IPV6_TUNNEL : name, IFNAMSIZ);
-  ifr.ifr_ifru.ifru_data = (void *) &p;
+	strscpy(ifr.ifr_name, target != NULL ? tunName : name, IFNAMSIZ);
+	ifr.ifr_ifru.ifru_data = p;
 
-  if ((err = ioctl(olsr_cnf->ioctl_s, target != NULL ? SIOCADDTUNNEL : SIOCDELTUNNEL, &ifr))) {
-    char buffer[INET6_ADDRSTRLEN];
+	if ((err = ioctl(olsr_cnf->ioctl_s, target != NULL ? SIOCADDTUNNEL : SIOCDELTUNNEL, &ifr))) {
+		olsr_syslog(OLSR_LOG_ERR, "Cannot %s tunnel %s to %s: %s (%d)\n", target != NULL ? "add" : "remove", name,
+				target != NULL ? inet_ntop(olsr_cnf->ip_version, target, buffer, sizeof(buffer)) : "-", strerror(errno),
+				errno);
+		return 0;
+	}
 
-    olsr_syslog(OLSR_LOG_ERR, "Cannot %s a tunnel %s to %s: %s (%d)\n",
-        target != NULL ? "add" : "remove", name,
-        target != NULL ? inet_ntop(olsr_cnf->ip_version, target, buffer, sizeof(buffer)) : "-",
-        strerror(errno), errno);
-    return 0;
-  }
-  return target != NULL ? if_nametoindex(name) : 1;
-}
-#endif
+	olsr_syslog(OLSR_LOG_INFO, "Tunnel %s %s, to %s", name, target != NULL ? "added" : "removed",
+			target != NULL ? inet_ntop(olsr_cnf->ip_version, target, buffer, sizeof(buffer)) : "-");
 
-/**
- * Dummy for generating an interface name for an olsr ipip tunnel
- * @param target IP destination of the tunnel
- * @param name pointer to output buffer (length IFNAMSIZ)
- */
-static void generate_iptunnel_name(union olsr_ip_addr *target, char *name) {
-  static char PREFIX[] = "tnl_";
-  static uint32_t counter = 0;
-
-  snprintf(name, IFNAMSIZ, "%s%08x", PREFIX,
-      olsr_cnf->ip_version == AF_INET ? target->v4.s_addr : ++counter);
+	return target != NULL ? if_nametoindex(name) : 1;
 }
 
 /**
  * demands an ipip tunnel to a certain target. If no tunnel exists it will be created
  * @param target ip address of the target
  * @param transportV4 true if IPv4 traffic is used, false for IPv6 traffic
+ * @param name pointer to name string buffer (length IFNAMSIZ)
  * @return NULL if an error happened, pointer to olsr_iptunnel_entry otherwise
  */
-struct olsr_iptunnel_entry *olsr_os_add_ipip_tunnel(union olsr_ip_addr *target, bool transportV4 __attribute__ ((unused))) {
+struct olsr_iptunnel_entry *olsr_os_add_ipip_tunnel(union olsr_ip_addr *target, bool transportV4 __attribute__ ((unused)), char *name) {
   struct olsr_iptunnel_entry *t;
 
   assert(olsr_cnf->ip_version == AF_INET6 || transportV4);
 
   t = (struct olsr_iptunnel_entry *)avl_find(&tunnel_tree, target);
   if (t == NULL) {
-    char name[IFNAMSIZ];
     int if_idx;
     struct ipaddr_str buf;
 
-    memset(name, 0, sizeof(name));
-    generate_iptunnel_name(target, name);
-
-    if (olsr_cnf->ip_version == AF_INET) {
-      if_idx = os_ip4_tunnel(name, &target->v4.s_addr);
-    }
-    else {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
-      if_idx = os_ip6_tunnel(name, &target->v6);
-#else
-      if_idx = 0;
-#endif
-    }
-
+    if_idx = os_ip_tunnel(name, (olsr_cnf->ip_version == AF_INET) ? (void *) &target->v4.s_addr : (void *) &target->v6);
     if (if_idx == 0) {
       // cannot create tunnel
       olsr_syslog(OLSR_LOG_ERR, "Cannot create tunnel %s\n", name);
@@ -250,14 +210,7 @@ struct olsr_iptunnel_entry *olsr_os_add_ipip_tunnel(union olsr_ip_addr *target,
     }
 
     if (olsr_if_set_state(name, true)) {
-      if (olsr_cnf->ip_version == AF_INET) {
-        os_ip4_tunnel(name, NULL);
-      }
-      else {
-  #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
-        os_ip6_tunnel(name, NULL);
-  #endif
-      }
+      os_ip_tunnel(name, NULL);
       return NULL;
     }
 
@@ -268,7 +221,7 @@ struct olsr_iptunnel_entry *olsr_os_add_ipip_tunnel(union olsr_ip_addr *target,
     memcpy(&t->target, target, sizeof(*target));
     t->node.key = &t->target;
 
-    strncpy(t->if_name, name, IFNAMSIZ);
+    strscpy(t->if_name, name, IFNAMSIZ);
     t->if_index = if_idx;
 
     avl_insert(&tunnel_tree, &t->node, AVL_DUP_NO);
@@ -282,6 +235,7 @@ struct olsr_iptunnel_entry *olsr_os_add_ipip_tunnel(union olsr_ip_addr *target,
  * Release an olsr ipip tunnel. Tunnel will be deleted
  * if this was the last user
  * @param t pointer to olsr_iptunnel_entry
+ * @param cleanup true to free t's memory
  */
 static void internal_olsr_os_del_ipip_tunnel(struct olsr_iptunnel_entry *t, bool cleanup) {
   if (!cleanup) {
@@ -296,14 +250,7 @@ static void internal_olsr_os_del_ipip_tunnel(struct olsr_iptunnel_entry *t, bool
   }
 
   olsr_if_set_state(t->if_name, false);
-  if (olsr_cnf->ip_version == AF_INET) {
-    os_ip4_tunnel(t->if_name, NULL);
-  }
-  else {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
-    os_ip6_tunnel(t->if_name, NULL);
-#endif
-  }
+  os_ip_tunnel(t->if_name, NULL);
 
   avl_delete(&tunnel_tree, &t->node);
   if (!cleanup) {
@@ -314,3 +261,4 @@ static void internal_olsr_os_del_ipip_tunnel(struct olsr_iptunnel_entry *t, bool
 void olsr_os_del_ipip_tunnel(struct olsr_iptunnel_entry *t) {
   internal_olsr_os_del_ipip_tunnel(t, false);
 }
+#endif /* __linux__ */
