@@ -14,12 +14,12 @@
 #define DEF_NIIT4TO6_IFNAME         "niit4to6"
 #define DEF_NIIT6TO4_IFNAME         "niit6to4"
 
-#ifdef linux
+#ifdef __linux__
 void olsr_init_niit(void);
 void olsr_setup_niit_routes(void);
 void olsr_cleanup_niit_routes(void);
 
 void olsr_niit_handle_route(const struct rt_entry *rt, bool set);
-#endif
+#endif /* __linux__ */
 
 #endif /* OLSR_NIIT_H_ */
